local const = require("const")
local ldbus = require("ldbus")
local json = require("json")

local watches_mappings = {
    all = "all",
    events = "event",
    properties = "property",
}

function is_not_empty_string(str)
    return type(str) == "string" and #str > 0
end

-- Watcher
local watcher = { conn = nil }

function watcher:new(o, conn)
    o = o or {}
    setmetatable(o, self)
    self.__index = self
    self.conn = conn
    return o
end

function watcher:add_watch(rule)
    if not rule then return "watch rule is null" end

    local message = ldbus.message.new_method_call(
            const.subscribe.wkn,
            const.subscribe.obj,
            const.subscribe.wkn,
            const.subscribe.interface_sub)
    if not message then
        return "create method call for subscribe failed"
    end

    local iter = ldbus.message.iter.new()
    message:iter_init_append(iter)
    iter:append_basic(rule)
    local reply, err_r = self.conn:send_with_reply_and_block(message, 10)
    if not reply then return "call subscribe failed: " .. tostring(err_r) end

    return nil
end

function watcher:remove_watch(rule)
    if not rule then return "watch rule is null" end

    local message = ldbus.message.new_method_call(
            const.subscribe.wkn,
            const.subscribe.obj,
            const.subscribe.wkn,
            const.subscribe.interface_unsub)
    if not message then
        return "create method call for unsubscribe failed"
    end

    local iter = ldbus.message.iter.new()
    message:iter_init_append(iter)
    iter:append_basic(rule)
    local reply = self.conn:send_with_reply_and_block(message, 10)
    if not reply then return "call unsubscribe failed" end

    return nil
end

function watcher:generate_watch_rule(things, receiver_name)
    local rule = { version = "1.0", content = {} }
    local fill_in_all_things = function()
        local source = { type = "device", data = "#" }
        local target = { type = receiver_name }
        local element = {}
        element["source"] = source
        element["target"] = target
        element["topicFilter"] = "all"
        table.insert(rule.content, element)
    end
    if not things then
        fill_in_all_things()
    elseif type(things) == "table" then
        repeat
            if #things == 0 then
                fill_in_all_things()
                break
            end
            for _, thing in pairs(things) do
                local product_key = thing["ProductKey"]
                if not is_not_empty_string(product_key) then
                    return nil, "ProductKey not a string or is empty"
                end

                local device_name = thing["DeviceName"]
                if not is_not_empty_string(device_name) then
                    return nil, "DeviceName is not a string or is empty"
                end

                local watches = thing["Watches"] or "all"
                if not is_not_empty_string(watches) then
                    return nil, "Watches not a string or is empty"
                end
                local topic = watches_mappings[watches]
                if not topic then
                    return nil, "Watches must be one of all, events, properties"
                end

                local source = {
                    type = "device",
                    data = "/" .. product_key .. "/" .. device_name
                }
                local target = { type = receiver_name }
                local element = {
                    source = source,
                    target = target,
                    topicFilter = topic
                }
                table.insert(rule.content, element)
            end
        until true
    else
        return nil, "illegal type for things: " .. type(things)
    end
    return json.encode(rule), nil
end

function watcher:destroy()
    ldbus.bus.release_name(conn, self.wkn)
end
-- Watcher End

return watcher
