#!/usr/bin/env python3

import sys
sys.path.insert(0, "..")
import logging
from datetime import datetime
import threading

try:
    from IPython import embed
except ImportError:
    import code

    def embed():
        vars = globals()
        vars.update(locals())
        shell = code.InteractiveConsole(vars)
        shell.interact()


from opcua import ua, uamethod, Server
import time

class SubHandler(object):

    """
    Subscription Handler. To receive events from server for a subscription
    """

    def datachange_notification(self, node, val, data):
        print("Python: New data change event", node, val)

    def event_notification(self, event):
        print("Python: New event", event)


# method to be exposed through server

def func(parent, variant):
    ret = False
    if variant.Value % 2 == 0:
        ret = True
    return [ua.Variant(ret, ua.VariantType.Boolean)]


# method to be exposed through server
# uses a decorator to automatically convert to and from variants

@uamethod
def multiply(parent, x, y):
    print("multiply method call with parameters: ", x, y)
    return x * y
    
@uamethod
def doublex(parent, x):
    print("doublex method call with parameters: ", x)
    return 2 * x

def task_datachange(var, mydev1Temp, myevgen, mysecondevgen, mydev1egen):
    import time

    count_float = 6.7
    count_int = 0
    time.sleep(5)

    while True:
        time.sleep(5)
        count_float += 1
        var.set_value(count_float)
        temp = mydev1Temp.get_value()
        mydev1Temp.set_value((temp+1)%80)

        myevgen.event.Message = ua.LocalizedText("MyFirstEvent %d" % count_int)
        myevgen.event.Severity = count_int
        myevgen.event.MyNumericProperty = count_int
        myevgen.event.MyStringProperty = "Property " + str(count_int)
        myevgen.trigger()
    
        mysecondevgen.trigger(message="MySecondEvent %d" % count_int)

        mydev1egen.event.Message = ua.LocalizedText("MyDev2Event %d" % count_int)
        mydev1egen.event.Severity = count_int
        mydev1egen.event.num = count_int * 2
        mydev1egen.event.string = "Property " + str(count_int*2)
        mydev1egen.trigger()

        count_int += 1

if __name__ == "__main__":
    # optional: setup logging
    logging.basicConfig(level=logging.WARN)
    #logger = logging.getLogger("opcua.address_space")
    # logger.setLevel(logging.DEBUG)
    #logger = logging.getLogger("opcua.internal_server")
    # logger.setLevel(logging.DEBUG)
    #logger = logging.getLogger("opcua.binary_server_asyncio")
    # logger.setLevel(logging.DEBUG)
    #logger = logging.getLogger("opcua.uaprocessor")
    # logger.setLevel(logging.DEBUG)
    server_num = sys.argv[1]
    device_num = sys.argv[2]
    print("server_num:",server_num, "device_num:", device_num)

    for i in range(0,int(server_num)):
        # now setup our server
        server = Server()
        #server.disable_clock()
        #server.set_endpoint("opc.tcp://localhost:4840/freeopcua/server/")
        server.set_endpoint("opc.tcp://0.0.0.0:"+str(4840+i)+"/freeopcua/server/")
        server.set_server_name("FreeOpcUa Example Server")

        # setup our own namespace
        uri = "http://examples.freeopcua.github.io"
        idx = server.register_namespace(uri)

        # create a new node type we can instantiate in our address space
        dev = server.nodes.base_object_type.add_object_type(0, "MyDevice")
        dev.add_variable(0, "sensor1", 1.0).set_modelling_rule(True)
        dev.add_property(0, "device_id", "0340").set_modelling_rule(True)
        ctrl = dev.add_object(0, "controller")
        ctrl.set_modelling_rule(True)
        ctrl.add_property(0, "state", "Idle").set_modelling_rule(True)

        # populating our address space

        # First a folder to organise our nodes
        myfolder = server.nodes.objects.add_folder(idx, "myEmptyFolder")

        # instanciate one instance of our device
        mydevice = server.nodes.objects.add_object(idx, "Device0001", dev)
        # get proxy to our device state variable
        mydevice_var = mydevice.get_child(["0:controller", "0:state"])

        # import some nodes from xml
        server.import_xml("custom_nodes.xml")

        # creating a default event object
        # The event object automatically will have members for all events properties
        # you probably want to create a custom event type, see other examples
        myevgen = server.get_event_generator()
        myevgen.event.Severity = 300

        # Creating a custom event: Approach 1
        # The custom event object automatically will have members from its parent (BaseEventType)
        etype = server.create_custom_event_type(idx, 
                                                'MyFirstEvent', 
                                                ua.ObjectIds.BaseEventType,
                                                [('MyNumericProperty', ua.VariantType.Float), 
                                                 ('MyStringProperty', ua.VariantType.String)])

        # Creating a custom event: Approach 2
        custom_etype = server.nodes.base_event_type.add_object_type(2, 'MySecondEvent')
        custom_etype.add_property(2, 'MyIntProperty', ua.Variant(0, ua.VariantType.Int32))
        custom_etype.add_property(2, 'MyBoolProperty', ua.Variant(True, ua.VariantType.Boolean))

        dev2etype = server.create_custom_event_type(idx, 
                                                    'MyDev2Event', 
                                                    ua.ObjectIds.BaseEventType,
                                                    [('num', ua.VariantType.UInt32), 
                                                     ('string', ua.VariantType.String)])

        for j in range(0, int(device_num)):
            # create directly some objects and variables
            mydev1 = server.nodes.objects.add_object(idx, "Device#"+str(j))
            myvar = mydev1.add_variable(idx, "MyVariable", 6.7)
            myvar.set_writable()    # Set MyVariable to be writable by clients
            myvar2 = mydev1.add_variable(idx, "MyVariable2", 6.7)
            myvar2.set_writable()    # Set MyVariable to be writable by clients
            mystringvar = mydev1.add_variable(idx, "MyStringVariable", "Really nice string")
            mystringvar.set_writable()    # Set MyVariable to be writable by clients
            myprop = mydev1.add_property(idx, "myproperty", "I am a property")
            multiply_node = mydev1.add_method(idx, "multiply", multiply, [ua.VariantType.Int64, ua.VariantType.Int64], [ua.VariantType.Int64])
            mydev1Temp = mydev1.add_variable(idx, "Temp", 20.0)
            mydev1Temp.set_writable()    # Set MyVariable to be writable by clients
            mydev1Banner = mydev1.add_variable(idx, "banner", "Alibaba device")
            mydev1Banner.set_writable()    # Set MyVariable to be writable by clients
            doublex_node = mydev1.add_method(idx, "doublex", doublex, [ua.VariantType.Int64],
                                              [ua.VariantType.Int64])

            myevgen = server.get_event_generator(etype, mydev1)
            mysecondevgen = server.get_event_generator(custom_etype, mydev1)
            mydev1egen = server.get_event_generator(dev2etype, mydev1)
            mydevice_var.set_value("Running")
            myevgen.trigger(message="This is BaseEvent")

            t = threading.Thread(target=task_datachange, args=(myvar, mydev1Temp, myevgen, mysecondevgen, mydev1egen))
            t.start()

        # starting!
        server.start()
        print("Available loggers are: ", logging.Logger.manager.loggerDict.keys())

    while True:
        time.sleep(5)
